require 'json'

# == Schema Information
#
# Table name: benchmark_batches
#
#  id               :integer          not null, primary key
#  performed_at     :datetime
#  created_at       :datetime
#  updated_at       :datetime
#  configuration_id :integer
#

class BenchmarkBatch < ActiveRecord::Base

  has_many :benchmark_results, dependent: :destroy, inverse_of: :benchmark_batch
  belongs_to :benchmark_configuration, inverse_of: :benchmark_batches
  has_and_belongs_to_many :tags

  validates_presence_of :performed_at

  def result_for(info)
    benchmark_results.where(benchmark_info: info).first
  end

  def all_benchmark_infos
    benchmark_results.map(&:benchmark_info)
  end

  def BenchmarkBatch.archive_directory
    archive_dir = File.join(Rails.root, 'public', 'archive')
    if (not File.exist? archive_dir) then
      Dir.mkdir(archive_dir)
    end
    return archive_dir
  end

  # Returns full path to archive file
  def archive_path
    return File.join(BenchmarkBatch.archive_directory, self.archive_filename)
  end

  def to_json()
    return JSON.parse( IO.read( self.archive_path ) )
  end

  def BenchmarkBatch.from_json(data)
    archive_name = "#{data["timestamp"]}_#{(data["tags"] || data["name"] || 'default').split(",").join('-')}_#{data["configuration"]["language"]}_#{data["configuration"]["os"]}_#{data["configuration"]["runtime"]}.json"
    archive_name = archive_name.tr(' :/', '___')

    # Step 2: store data to database for faster access
    # ------------------------------------------------
    batch = BenchmarkBatch.create!(
      benchmark_configuration: BenchmarkConfiguration.from_json(data['configuration']),
      performed_at: DateTime.parse(data["timestamp"]),
      archive_filename: archive_name)

    data["outcomes"].each do | outcome |


      bench_info = BenchmarkInfo.from_json(outcome["benchmark"])

      result = BenchmarkResult.create!(
        benchmark_info_id: bench_info.id,
        benchmark_batch_id: batch.id
      )

      outcome["times"].each do |duration|
        result.benchmark_durations.create!(duration: duration)
      end

      outcome["parameters"].each do | name, value|
        result.benchmark_parameters.create!(name: name, value: value)
      end

    end

    (data["tags"] || data["name"] || 'default').split(",").each do | tag |
      batch.tags.append(Tag.find_by_name(tag) || Tag.create!(name: tag))
    end

    File.open( batch.archive_path, 'w') { |file| file.write(JSON.pretty_generate(data))  }

    return batch
  end


  def performed_at_ensured
    if performed_at.present?
      performed_at
    else
      Date.today
    end
  end
end
